/*##########################Zustandsfunktionen##########################*/

/*
 * Fahr-Funktion
 * Lässt den Roboter "random" auf dem Tisch fahren 
 * Stößt er auf einen Abgrund, so dreht er sich und wechselt in den Suchmodus
 */
ZustandsTyp FahreAufTisch(void)
{
  //Serial.println("FahreAufTisch");
  ZustandsTyp rueck;  //Variable für die Rückgabe des neuen Zustandes

  //Der Roboter bewegt sich auf dem Tisch, bis der Kantensensor Alarm schlägt
  if (TestSensorVorne()==false)
  {
    Shotti.bewegeGerade('v', 0.7);
    rueck = Auto;
  }
  else
  {
    Shotti.bewegeGeradeStrecke('r', 0.7, 50);
    int vz = random(0,1);
    if(vz==0)
      {vz=-1;}
    Shotti.bewegeDreh(vz * random(80, 120), 0.7);
    if(TestSensorVorne()==true) //Wenn immer noch am Abgrund, dann kein Suchlauf
    {
      rueck = Auto;
    }
    else                        //Sonst wird der Zustand zu Suchen geändert
    {
      rueck = Suchen;
    }
  }
  return rueck;
}

/*
 * Suchlauf-Funktion
 * Führt einen Suchlauf mit dem Sonar aus und reagiert auf das Ergebnis
 * Wurde ein Glas gefunden -> ausrichten, darauf zufahren, erneut suchen, dann Zustand "Auf Glas zufahren"
 * Wurde kein Glas gefunden -> Wechsele zu "Auto"
 */
ZustandsTyp Suchlauf(int range)
{
  ZustandsTyp rueck = ParallelSuche; //Default-Case: Kein Glas wird gefunden => Roboter fährt auf dem Tisch, bis zu einer Kante (o.ä.)
  float winkel, abstand;

  if(SucheGlas(90 - range/2, 90 + range/2, &winkel, &abstand) == true && TestSensorVorne() == false)
  {
    //Serial.println(winkel);
    Shotti.bewegeDreh(winkel, 0.5);
    if(abstand < 150) //Wenn der Abstand zum Glas <150mm ist, dann gib direkt Erfolgsmeldung
    {
      rueck = FahreZuGlas;
    }
    else  //Sonst führe eine zweite Messung aus
    {
      //Serial.print("Abstand: ");Serial.println(round((abstand/6.0)));
      Shotti.bewegeGeradeStrecke('v', 0.4, abstand/6.0); //Bewege dich ein Sechstel des gemessenen Abstandes auf das Glas zu und miss erneut
      if(SucheGlas(90 - range/4, 90 + range/4, &winkel, &abstand) == true && TestSensorVorne() == false)
      {
        if(abs(winkel) < 6) //Wenn der Winkel der bei der zweiten Messung entdeckt wird, weniger als x Grad von der ersten Messung abweicht, wurde ein Glas gefunden
        {
          Shotti.bewegeDreh(winkel, 0.3);
          rueck = FahreZuGlas;   //Zustand "Fahre auf Glas zu" nach zwei erfolgreichen Messungen
        }
      }
    }
  }
  return rueck;
}

/*
 * Funktion zum langsam auf das Glas zufahren
 */
ZustandsTyp FahreAufGlasZu(int *fuellmenge)
{
  ZustandsTyp rueck = Suchen;   //Rückgabewert der Funktion, default auf "Suchen"
  boolean endschalter = false;  //Variable für den Zustand des Endschalters
  
  while(endschalter == false && TestSensorVorne(20) == false) //Solange der Endschalter nicht betätigt und kein Abgrund gefunden ist...
  {
    endschalter = Endschalter();    //Endschalter überprüfen...
    Shotti.bewegeGerade('v', 0.2);  //und geradeaus fahren
  }
  
  if(endschalter == true) //Wenn der Schalter für Abbruch gesorgt hat => Glas befüllen...
  {
    Shotti.bewegeGeradeStrecke('v',0.2,10);
    delay(2000);                                //Warte zwei Sekunden vor dem Öffnen
    *fuellmenge = oeffneVentil(*fuellmenge);    //Gib Flüssigkeit aus
    delay(2000);                                //Warte zwei Sekunden bevor der Roboter Abstand vom Glas nimmt
    Shotti.bewegeGeradeStrecke('r', 0.5, 120);  //Fahre rückwärts
    rueck = Standby;                            //Wechsele in den Standby-Modus
  }
  else                    //Sonst ist Kante entdeckt und es wird auf "ParallelSuche" gestellt => Drehung
  {
    rueck = ParallelSuche;
  }

  return rueck;
}

/*
 * Standby-Funktion
 * Warte auf ein Aktivierungsevent. Gib "Standby" zurück, wenn nichts passiert, "Suche", wenn das event entdeckt wird
 */
ZustandsTyp standby()
{
  if(digitalRead(standbyPin) == HIGH)
  {
    Shotti.motorEnable(1);
    return Suchen;
  }
  else
  {
    Shotti.motorEnable(0);
    return Standby;
  }
}

/*
 * Parallele Fahr- und Suchfunktion -> nicht funktionstüchtig 
 * wegen doppelbestimmtheit bei 25cm (entspricht dem return bei "unendlicher" Entfernung)
 */
ZustandsTyp FahreSuche(void)
{
  ZustandsTyp rueck = ParallelSuche;  //Standardrückgabe -> Weiter FahreSuche() ausführen
  int range = 100;                    //Range, in der die parallele Suche stattfindet
  int winkel = 90 - range/2;          //winkel-Variable für den Servo
  float abstand = 300;                //Variable, für gemessenen Abstand
  unsigned long steps = 0;            //Schrittzähler für die stepper
  int wdhSteps = 8;                   //Nach wie vielen Schritten die nächste Messung ausgeführt wird
  int richtung = -1;                  //Drehrichtung des Servos
  boolean abgrund = TestSensorVorne();//Test, ob ein Abgrund angefahren wird
  int abstand_krit = 150;             //Abstand, ab dem die Funktion von einem Gegenstand ausgeht

  while(abgrund == false && (abstand > abstand_krit || abstand < 0))
  {
    abgrund = TestSensorVorne();
    Shotti.bewegeGerade('v', 0.7);
    if(steps < Shotti.returnStepCount('r'))
    {
      steps = Shotti.returnStepCount('r') + wdhSteps;
      if(richtung == -1)
        {winkel++;}
      else if(richtung == 1)
        {winkel--;}
      abstand = SuchWinkel(winkel);
      //Serial.println(abstand);

      if(winkel < 90-range/2 || winkel > 90+range/2)
        {
          //Serial.println("Aendere Richtung");
          richtung *= -1;
        }
    }
  }
  
  if(abgrund == true)
  {
    //Serial.println("ABGRUND!");
    Shotti.bewegeGeradeStrecke('r', 0.7, 50);
    Shotti.bewegeDreh(random(50, 60), 0.7);
    if(TestSensorVorne()==true) //Wenn immer noch am Abgrund, dann kein Suchlauf
    {
      rueck = ParallelSuche;
    }
    else                        //Sonst wird der Zustand zu Suchen geändert
    {
      rueck = Suchen;
    }
  }
  else
  {
    rueck = Suchen;
  }
  
  return rueck;
}

//Gehört zur FahreSuche-Funktion, ermittelt Messwert für einen bestimmten Winkel
float SuchWinkel(int winkel)
{
  //Serial.println(winkel);
  servo.write(winkel);

  int durchlaeufe = 5;
  float Mess=0;
  float maxi = SensorGlas.returnLimit('l'), mini = SensorGlas.returnLimit('u');
  float temp;
  int test = 0; //Wie viele gültige Werte wurden gemessen?
  
  for (int i = 0; i < durchlaeufe; i++)
  {
    temp = SensorGlas.messen_mm();
    if(temp > SensorGlas.returnLimit('l') && temp < SensorGlas.returnLimit('u'))
      {
        Mess += temp;
        test++;
        //Serial.println(temp);
        if(temp < mini)
          {mini = temp;}
        if(temp > maxi)
          {maxi = temp;}
      }
    //else
      //durchlaeufe++;
  }

  //Serial.print("Abstand  ");Serial.println(Mess/5);
  if(abs(maxi - mini) > 30 && test < 3)
    {return -1;}  //Ungültige Messung
  else
    {return Mess/test;}
  
  //return SensorGlas.messen_mm();
}
