/*##################Einbinden der benötigten Klassen und Bibliotheken##################*/
#include "stepper.h"      //Klasse für die stepper-Motoren
#include "robot.h"        //Klasse für den Roboter
#include "lichtsensor.h"  //Klasse für die Lichtsensoren
#include <Servo.h>        //Servo-Bibliothek zur Ansteuerung des Servos
boolean TestSensorVorne(int delta = 10);  //Deklaration dieser Funktion, nötig zur Verwendung von default arguments

/*##################Deklaration/Initialisierung der benötigten Objekte und globaler Variablen##################*/
//Deklaration des Roboterobjekts
Robot Shotti;

//Deklaration der drei Lichtsensoren
LichtSensor SensorVorne; //Deklaration des Fallsensors vorne am Roboter
LichtSensor SensorGlas;  //Deklaration des Glassensors auf dem Servo

//Deklaration des Servoobjekts
Servo servo;
int const servoPin = 10; //Digitalpin für die Servosteuerung

//Deklaration des Magnetventil-Pins
int const ventilPin = 8;

//Deklaration des Endschalter-Pins und des Standby-Schalter-Pins
int const endschalterPin = 9;
int const standbyPin = 11;

//Enum-Variable für die Zustandsautomatisierung und Initialisierung des Zustandes
enum ZustandsTyp {
  Standby,
  Auto,
  FahreZuGlas,
  Suchen,
  ParallelSuche
};
ZustandsTyp aktuellerZustand = Suchen;

/*##################Setup-Funktion, Initialisierung der Objekte##################*/
void setup()
{
  //Setup des Roboters (Initialisierung der Parameter)
  Shotti.setup(45, 140, 200, 16); //Radradius, Achsenradius (Drehradius), Länge, microstep-Faktor

  //Initialisierung der Serial-Kommunikation mit 115200 Baud (für Testausgaben)
  Serial.begin(115200);

  //Initialisierung der beiden Abstandssensoren, Pinbelegegungen und Messgrenzen werden festgelegt
  SensorVorne.setup(7, 40, 300);  //Sensor vorne (nach unten gerichtet, für Tischsicherung)
  SensorGlas.setup(6, 100, 800);  //Richtungssensor für das Glas

  //Lichtsensor-Schwelle initialisieren
  SensorVorne.SchwelleErmitteln();
  //Serial.print("SensorVorne:  ");Serial.println(SensorVorne.Schwelle);

  //Initialisierung des Servos
  servo.attach(servoPin, 800, 2100); //Servo-Pin, min und max Pulslänge
  servo.write(90);
  delay(3000);

  //Initialisierung des MagnetventilPins auf OUTPUT
  pinMode(ventilPin, OUTPUT);
  digitalWrite(ventilPin, LOW);
}

/*##################Loop-Funktion, Ausführung##################*/
void loop()
{ 
  int fuellmenge = 700; //Füllstand des Tanks in ml, initial 700 ml

  while(true)
  {
    //Serial.println(aktuellerZustand);

    switch (aktuellerZustand)
    {
      case Auto:
        long stepcount;
        stepcount = Shotti.returnStepCount('r') + 3000;
        while(aktuellerZustand == Auto && Shotti.returnStepCount('r') < stepcount)
        {
          aktuellerZustand = FahreAufTisch();
        }
        if(Shotti.returnStepCount('r') >= stepcount){aktuellerZustand = Suchen;}
        break;
      case Suchen:
        aktuellerZustand = Suchlauf(120); //Suchlauf in 120Grad range
        break;
      case FahreZuGlas:
        aktuellerZustand = FahreAufGlasZu(&fuellmenge);
        break;
      case Standby:
        aktuellerZustand = standby();
        break;
      case ParallelSuche:
        aktuellerZustand = FahreSuche();
        break;
    }
  }
}
