#include "Arduino.h"

/*
   Klasse für Lichtsensoren
   Enthält Initialisierungs- und Messfunktionen
*/

class LichtSensor
{
  private:
    int Messpin;  //Pin an dem der Sensor gemessen wird
    int dist_min; //Minimal messbarer Abstand
    int dist_max; //Maximal messbarer Abstand

  public:
    int   Schwelle;               //Prozentualer Wert des Lichtsensors bei normaler Tischposition, public, damit aus dem Hauptprogramm darauf zugegriffen werden kann
    void  setup(int newMesspin, int _dist_min, int _dist_max);  //Setup-Funktion
    float messen(void);           //Mess-Funktion, gibt prozentualen Messwert zurück
    float messenRAW(void);        //Gibt Rohmessdaten zurück
    float messen_mm(void);        //Gibt Messdaten in mm zurück (nur für 10-80cm-Sensor kalibriert)
    float messen_mm_MW(void);     //Mittelwertbildung bei der Messung
    void  SchwelleErmitteln();    //Ermittelt den Standardabstand bei Initialisierung
    int   returnLimit(char L);    //Gibt dist_min oder dist_max zurück
};


/*#################Definitionen der Klassenfunktionen Lichtsensoren#####################*/
/*##################Setup-Funktion zur Initialisierung von neuen Lichtsensoren##################*/
void LichtSensor::setup(int newMesspin, int _dist_min, int _dist_max)
{
  Messpin = newMesspin;
  dist_min = _dist_min;
  dist_max = _dist_max;
  Schwelle = 0;
}

/*
  ##################Funktion zum Auslesen der Lichtsensoren (Prozent)##################
  Prozentualer Messwert des Lichtsensors (range 0 - 100)
*/
float LichtSensor::messen(void)
{
  return map(analogRead(Messpin), 0, 1023, 0, 100);
}

/*
  ##################Funktion zum Auslesen der Lichtsensoren (Rohwert (range 0 - 1023))##################
*/
float LichtSensor::messenRAW(void)
{
  return analogRead(Messpin);//map(analogRead(Messpin), 0, 1023, 0, 5000);
}

/*
   ##################Funktion zum Auslesen der Lichtsensoren (in mm)##################
   Wert der Messung umgerechnet in mm (über lineare Approximation -> s. Kalibrier-spreadsheet)
   Kalibriert für Sharp-Sensor mit range 10-80cm
*/
float LichtSensor::messen_mm(void)
{
  float mess = map(analogRead(Messpin), 0, 1023, 0, 5000);
  float d_i = ((mess / 1000) - 0.1794) / 22.358;
  return 10 / d_i;
}

/*
   ##################Funktion zum Auslesen der Lichtsensoren (in mm)##################
   Wie vorher, nur nun mit Mittelwertbildung und Bennenung ungültiger Messungen
*/
float LichtSensor::messen_mm_MW(void)
{
  int durchlaeufe = 5;
  float mess = 0;
  float maxi = dist_min, mini = dist_max;
  float temp;
  int test = 0; //Wie viele gültige Werte wurden gemessen?

  for (int i = 0; i < durchlaeufe; i++)
  {
    temp = messen_mm();
    if (temp > dist_min && temp < dist_max)
    {
      mess += temp;
      test++;
      //Serial.println(temp);
      if (temp < mini)
      {
        mini = temp;
      }
      if (temp > maxi)
      {
        maxi = temp;
      }
    }
//    delayMicroseconds(50);
//    Serial.println(temp);
  }
//  Serial.println("MaxMin");
//  Serial.println(maxi);
//  Serial.println(mini);
//  delay(2000);
  
  if (abs(maxi - mini) > 60 || test < 5)
  {
    return dist_max; //Ungültige Messung
  }
  else
  {
    return (float)mess / test; //Gültige Messung
  }
}


/*
  ##################Bestimmung des Schwellwertes der Fallsicherung##################
*/
void LichtSensor::SchwelleErmitteln()
{
  int messungen = 5;  //Anzahl der Mesungen, über die gemittelt werden soll

  for (int k = 0; k < messungen; k++) //for-Schleife zur Summenbildung
  {
    Schwelle += messen(); //Aufsummierung der Messwerte
    //Serial.println(Schwelle); //Ausgabe der Messwerte
  }
  Schwelle /= messungen;  //Ermittlung des arithmetischen Mittels
}

/*
  ##################Gibt die Messgrenzen des Lichtsensors zurück##################
*/
int   LichtSensor::returnLimit(char L)
{
  if (L == 'u')      return dist_max;
  else if (L == 'l') return dist_min;
}
