#include "Arduino.h"

/*
 * Klasse zur Ansteuerung der Stepper-Motoren
 * Enthält Variablen und Funktionen, mit denen die stepper-Motoren angesteuert werden können
 */

class StepperMotor
{
  private:
    int  ena, stp, dir;   //Die drei Pins für Stepper: enable, step, direction
    long repeatTime;      //Wiederholzeitpunkt für die Stepper-Pulse
    unsigned long steps;  //step-Zähler, der jeden Step der vom Motor ausgeführt werden soll zählt
    long t_beschl;        //Zeitvariable, um die Beschleunigung zu berechnen
    int  dir_curr;        //Aktuelle Drehrichtung des Motors

  public:
    float v, v_p;       //Die aktuelle Geschwindigkeit und die vergangene dieses Motors, public, damit sie immer eingesehen werden kann.
    int fullturn;       //steps, die für eine volle Raddrehung benötigt werden (normal 200, 16tel microstepping -> 3200)
    void setup(int newEna, int newStp, int newDir, int newFullturn); //setup-Funktion zur Verknüpfung der Hardware
    int  setV(float geschw, int richtung); //setV-Funktion zum Einstellen der Motorgeschwindigkeit
    void beschleunige(float geschw, int richtung);
    void motorEnable(int OnOff);

    int outputSteps();
};


/*#################Definitionen der Klassenfunktionen Stepper-Motoren#####################*/
/*
 * ##################setup-Funktion für die Stepper-Motoren##################
 * Getrennte setup-Funktion, nötig, um Hardware zu initialisieren, da die Konstruktoren global deklarierter Objekte vor der Hardware-initialisierung aufgerufen werden.
 */
void StepperMotor::setup(int newEna, int newStp, int newDir, int newFullturn)
{
  //Zuweisung der eingegebenen Werte auf die Pinvariablen und microstepping
  ena = newEna;
  stp = newStp;
  dir = newDir;
  fullturn = newFullturn;

  //initialisierung der Pins auf OUTPUT.
  pinMode(ena, OUTPUT);
  pinMode(stp, OUTPUT);
  pinMode(dir, OUTPUT);
  digitalWrite(ena, LOW); //Enable auf LOW setzen, damit Strom AN(5V)

  repeatTime = 0; //repaeatTime auf 0 initialisieren, damit der erste Motorschritt funktioniert
  steps = 0;      //gezählte steps auf 0 setzen, so dass vernünftig aufsummiert werden kann
  v = 0;          //geschw am Anfang ist 0
  t_beschl = 0;   //Initialisierung der Beschleunigungszeit
}

/*
 * ##################setV-Funktion für die Stepper-Motoren##################
 * geschw prozentual zwischen 0 und 1, Richtung 0 oder 1
 */
int StepperMotor::setV(float geschw, int richtung)
{
  int del, bewegt = 0;
  float thresh = 0.05; //Grenze für Stillstand

  v = geschw;
  dir_curr = richtung;
  geschw = geschw * 100;
  switch (fullturn)
  {
    case 200:
      del = map(geschw, 0, 100, 50000, 10000); //Mappe die Geschwindigkeit in Prozent auf das Delay in Mikrosekunden (Pause zwischen den Pulsen)
      break;
    case 3200:
      del = map(geschw, 0, 100, 3125, 625); //Mappe die Geschwindigkeit in Prozent auf das Delay in Mikrosekunden (Pause zwischen den Pulsen)
  }

  if (richtung == 0)
  {
    digitalWrite(dir, HIGH); // Set Dir high
  }
  else if (richtung == 1)
  {
    digitalWrite(dir, LOW); // Set Dir high
  }


  if (repeatTime <= micros() && geschw >= thresh)
  {
    digitalWrite(stp, HIGH); // Output high
    delayMicroseconds(1); // Sende Impuls für eine Mikrosekunde
    digitalWrite(stp, LOW); // Output low
    repeatTime = micros() + del;
    bewegt = 1; //return, ob ein step gemacht wurde
    steps += 1; //Gesamtanzahl steps vom Motor hochsetzen
  }
  return bewegt;
}

/*
 * ##################Beschleunigungsfunktion - nicht final/funktional##################
 */
void StepperMotor::beschleunige(float geschw, int richtung)
{
  float a = 0.00001; //beschl
  int dt;

  int test;
  //Serial.println("test");

  /*if(richtung != dir_curr)
    {

    }*/

  if (abs(v - geschw) < .02)
  {
    v = geschw;
    t_beschl = millis();
  }
  else if (v < geschw)
  {
    dt = millis() - t_beschl;
    v = v + a * dt;
  }
  else if (v > geschw)
  {
    dt = millis() - t_beschl;
    v = v - a * dt;
  }

  setV(v, richtung);
}

void StepperMotor::motorEnable(int OnOff)
{
  if(OnOff == 0)
    digitalWrite(ena, HIGH);  //Motor aus
  else if(OnOff == 1)
    digitalWrite(ena, LOW); //Motor an
}

/*
 * ##################Funktion zur Ausgabe der getätigten steps dieses Motors##################
 */
int StepperMotor::outputSteps()
{
  return (steps);
}
